package org.jboss.cache.api.nodevalidity;

import org.jboss.cache.Cache;
import org.jboss.cache.CacheFactory;
import org.jboss.cache.UnitTestCacheFactory;
import org.jboss.cache.config.Configuration.NodeLockingScheme;
import org.jboss.cache.util.TestingUtil;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

/**
 * @author <a href="mailto:manik AT jboss DOT org">Manik Surtani</a>
 * @since 2.1.0
 */
@Test(groups = {"functional", "pessimistic"}, testName = "api.nodevalidity.LocalPessNodeValidityTest")
public class LocalPessNodeValidityTest extends NodeValidityTestBase
{
   private ThreadLocal<Cache<String, String>> cacheTL = new ThreadLocal<Cache<String, String>>();

   public LocalPessNodeValidityTest()
   {
      clustered = false;
      nodeLockingScheme = NodeLockingScheme.PESSIMISTIC;
   }

   @AfterMethod
   public void tearDown()
   {
      Cache<String, String> cache = cacheTL.get();
      super.tearDown();
      TestingUtil.killCaches(cache);
      cacheTL.set(null);
   }

   protected Cache<String, String> createObserver()
   {
      return createModifier();
   }

   protected Cache<String, String> createModifier()
   {
      Cache<String, String> cache = cacheTL.get();
      
      if (cache == null)
      {
         CacheFactory<String, String> f = new UnitTestCacheFactory<String, String>();
         cache = f.createCache(false);
         nodeLockingSchemeSpecificSetup(cache.getConfiguration());
         cache.start();
         cacheTL.set(cache);
         return cache;
      }
      return cache;
   }
}
